
#include "extending.h"


void machine::create_predefined_variables()
{
    chomik::machine::create_predefined_variables();

    chomik::generic_name gn;
    gn.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("the"));
    gn.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("created"));
    gn.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("my"));
    gn.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("class"));
    gn.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("index"));
    std::shared_ptr<chomik::signature> the_created_my_class_index=std::make_shared<chomik::signature>(gn);
    add_variable_with_value(std::make_shared<chomik::simple_variable_with_value_integer>(std::move(the_created_my_class_index), 0));
}


bool machine::get_is_user_defined_executable(const chomik::signature & s) const
{
    if (s.get_vector_of_items().size() == 5)
    {
        if (s.get_vector_of_items()[0]->get_it_is_identifier("create")
            && s.get_vector_of_items()[1]->get_it_is_identifier("new")
            && s.get_vector_of_items()[2]->get_it_is_identifier("my")
            && s.get_vector_of_items()[3]->get_it_is_identifier("class")
            && s.get_vector_of_items()[4]->get_it_is_string())
        {
            return true;
        }
    }

    return false;
}


void machine::execute_user_defined_executable(const chomik::signature & s)
{
    if (s.get_vector_of_items().size() == 5)
    {
        if (s.get_vector_of_items()[0]->get_it_is_identifier("create")
            && s.get_vector_of_items()[1]->get_it_is_identifier("new")
            && s.get_vector_of_items()[2]->get_it_is_identifier("my")
            && s.get_vector_of_items()[3]->get_it_is_identifier("class")
            && s.get_vector_of_items()[4]->get_it_is_string())
        {
            add_my_class(std::make_unique<my_class>(s.get_vector_of_items()[4]->get_value_string()));

            chomik::generic_name gn2;
            gn2.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("the"));
            gn2.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("created"));
            gn2.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("my"));
            gn2.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("class"));
            gn2.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("index"));
            chomik::signature the_created_my_class_index{gn2};
            get_variable_with_value(the_created_my_class_index).assign_value_integer(get_last_created_my_class_index());
        }
    }
}

chomik::program the_program;
chomik::parser the_parser{the_program};


int main(int argc, char *arv[])
{
    int return_value = 0;
    std::string filename="a_valid_path_to_the_chomik_code/some_chomik_code.chomik";

    chomik::parser::register_parser(&the_parser);

    if (the_parser.parse(filename.c_str()) == 0)
    {
            machine m;
            m.create_predefined_types();
            m.create_predefined_variables();
            m.create_predefined_streams();

            the_program.execute(m);


            chomik::generic_name gn;
            gn.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("the"));
            gn.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("program"));
            gn.add_generic_name_item(std::make_shared<chomik::identifier_name_item>("return"));
            chomik::signature s0{gn};

            return_value = m.get_variable_value_integer(s0);
    }

    return return_value;
}
